from sympy.core import Add, C, Derivative, Dummy, Expr, S, sympify, Wild
from sympy.concrete.gosper import gosper_sum
from sympy.functions.elementary.piecewise import piecewise_fold
from sympy.polys import apart, PolynomialError
from sympy.solvers import solve

def _free_symbols(function, limits):
    """Helper function to return the symbols that appear in a sum-like object
    once it is evaluated.
    """
    isyms = function.free_symbols
    for xab in limits:
        # take out the target symbol
        if xab[0] in isyms:
            isyms.remove(xab[0])
        # add in the new symbols
        for i in xab[1:]:
            isyms.update(i.free_symbols)
    return isyms

class Sum(Expr):
    """Represents unevaluated summation."""

    __slots__ = ['is_commutative']

    def __new__(cls, function, *symbols, **assumptions):
        from sympy.integrals.integrals import _process_limits

        # Any embedded piecewise functions need to be brought out to the
        # top level so that integration can go into piecewise mode at the
        # earliest possible moment.
        function = piecewise_fold(sympify(function))

        if function is S.NaN:
            return S.NaN

        if not symbols:
            raise ValueError("Summation variables must be given")

        limits, sign = _process_limits(*symbols)

        # Only limits with lower and upper bounds are supported; the indefinite Sum
        # is not supported
        if any(len(l) != 3 or None in l for l in limits):
            raise ValueError('Sum requires values for lower and upper bounds.')

        obj = Expr.__new__(cls, **assumptions)
        arglist = [sign*function]
        arglist.extend(limits)
        obj._args = tuple(arglist)
        obj.is_commutative = function.is_commutative # limits already checked

        return obj

    @property
    def function(self):
        return self._args[0]

    @property
    def limits(self):
        return self._args[1:]

    @property
    def variables(self):
        """Return a list of the summation variables

        >>> from sympy import Sum
        >>> from sympy.abc import x, i
        >>> Sum(x**i, (i, 1, 3)).variables
        [i]
        """
        return [l[0] for l in self.limits]

    @property
    def free_symbols(self):
        """
        This method returns the symbols that will exist when the
        summation is evaluated. This is useful if one is trying to
        determine whether a sum depends on a certain symbol or not.

        >>> from sympy import Sum
        >>> from sympy.abc import x, y
        >>> Sum(x, (x, y, 1)).free_symbols
        set([y])
        """
        if self.function.is_zero:
            return set()
        return _free_symbols(self.function, self.limits)

    @property
    def is_zero(self):
        """A Sum is only zero if its function is zero or if all terms
        cancel out. This only answers whether the summand zero."""

        return self.function.is_zero

    @property
    def is_number(self):
        """
        Return True if the Sum will result in a number, else False.

        sympy considers anything that will result in a number to have
        is_number == True.

        >>> from sympy import log
        >>> log(2).is_number
        True

        Sums are a special case since they contain symbols that can
        be replaced with numbers. Whether the integral can be done or not is
        another issue. But answering whether the final result is a number is
        not difficult.

        >>> from sympy import Sum
        >>> from sympy.abc import x, y
        >>> Sum(x, (y, 1, x)).is_number
        False
        >>> Sum(1, (y, 1, x)).is_number
        False
        >>> Sum(0, (y, 1, x)).is_number
        True
        >>> Sum(x, (y, 1, 2)).is_number
        False
        >>> Sum(x, (y, 1, 1)).is_number
        False
        >>> Sum(x, (x, 1, 2)).is_number
        True
        >>> Sum(x*y, (x, 1, 2), (y, 1, 3)).is_number
        True
        """

        return self.function.is_zero or not self.free_symbols

    def doit(self, **hints):
        #if not hints.get('sums', True):
        #    return self
        f = self.function
        for limit in self.limits:
            i, a, b = limit
            dif = b - a
            if dif.is_Integer and dif < 0:
                a, b = b, a

            f = eval_sum(f, (i, a, b))
            if f is None:
                return self

        if hints.get('deep', True):
            return f.doit(**hints)
        else:
            return f

    def _eval_summation(self, f, x):
        return

    def _eval_derivative(self, x):
        """
        Differentiate wrt x as long as x is not in the free symbols of any of
        the upper or lower limits.

        Sum(a*b*x, (x, 1, a)) can be differentiated wrt x or b but not `a`
        since the value of the sum is discontinuous in `a`. In a case
        involving a limit variable, the unevaluated derivative is returned.
        """

        # diff already confirmed that x is in the free symbols of self, but we
        # don't want to differentiate wrt any free symbol in the upper or lower
        # limits
        # XXX remove this test for free_symbols when the default _eval_derivative is in
        if x not in self.free_symbols:
            return S.Zero

        # get limits and the function
        f, limits = self.function, list(self.limits)

        limit = limits.pop(-1)

        if limits: # f is the argument to a Sum
            f = Sum(f, *limits)

        if len(limit) == 3:
            _, a, b = limit
            if x in a.free_symbols or x in b.free_symbols:
                return None
            df = Derivative(f, x, **{'evaluate': True})
            rv = Sum(df, limit)
            if limit[0] not in df.free_symbols:
                rv = rv.doit()
            return rv
        else:
            return NotImplementedError('Lower and upper bound expected.')

    def euler_maclaurin(self, m=0, n=0, eps=0, eval_integral=True):
        """
        Return an Euler-Maclaurin approximation of self, where m is the
        number of leading terms to sum directly and n is the number of
        terms in the tail.

        With m = n = 0, this is simply the corresponding integral
        plus a first-order endpoint correction.

        Returns (s, e) where s is the Euler-Maclaurin approximation
        and e is the estimated error (taken to be the magnitude of
        the first omitted term in the tail):

            >>> from sympy.abc import k, a, b
            >>> from sympy import Sum
            >>> Sum(1/k, (k, 2, 5)).doit().evalf()
            1.28333333333333
            >>> s, e = Sum(1/k, (k, 2, 5)).euler_maclaurin()
            >>> s
            -log(2) + 7/20 + log(5)
            >>> from sympy import sstr
            >>> print(sstr((s.evalf(), e.evalf()), full_prec=True))
            (1.26629073187415, 0.0175000000000000)

        The endpoints may be symbolic:

            >>> s, e = Sum(1/k, (k, a, b)).euler_maclaurin()
            >>> s
            -log(a) + log(b) + 1/(2*b) + 1/(2*a)
            >>> e
            Abs(-1/(12*b**2) + 1/(12*a**2))

        If the function is a polynomial of degree at most 2n+1, the
        Euler-Maclaurin formula becomes exact (and e = 0 is returned):

            >>> Sum(k, (k, 2, b)).euler_maclaurin()
            (b**2/2 + b/2 - 1, 0)
            >>> Sum(k, (k, 2, b)).doit()
            b**2/2 + b/2 - 1

        With a nonzero eps specified, the summation is ended
        as soon as the remainder term is less than the epsilon.
        """
        m = int(m)
        n = int(n)
        f = self.function
        assert len(self.limits) == 1
        i, a, b = self.limits[0]
        s = S.Zero
        if m:
            for k in range(m):
                term = f.subs(i, a+k)
                if (eps and term and abs(term.evalf(3)) < eps):
                    return s, abs(term)
                s += term
            a += m
        x = Dummy('x')
        I = C.Integral(f.subs(i, x), (x, a, b))
        if eval_integral:
            I = I.doit()
        s += I
        def fpoint(expr):
            if b is S.Infinity:
                return expr.subs(i, a), 0
            return expr.subs(i, a), expr.subs(i, b)
        fa, fb = fpoint(f)
        iterm = (fa + fb)/2
        g = f.diff(i)
        for k in range(1, n+2):
            ga, gb = fpoint(g)
            term = C.bernoulli(2*k)/C.factorial(2*k)*(gb-ga)
            if (eps and term and abs(term.evalf(3)) < eps) or (k > n):
                break
            s += term
            g = g.diff(i, 2)
        return s + iterm, abs(term)

    def _eval_subs(self, old, new): # XXX this should be the same as Integral's
        if any(old == v for v in self.variables):
            return self

def summation(f, *symbols, **kwargs):
    r"""
    Compute the summation of f with respect to symbols.

    The notation for symbols is similar to the notation used in Integral.
    summation(f, (i, a, b)) computes the sum of f with respect to i from a to b,
    i.e.,

    ::

                                    b
                                  ____
                                  \   `
        summation(f, (i, a, b)) =  )    f
                                  /___,
                                  i = a

    If it cannot compute the sum, it returns an unevaluated Sum object.
    Repeated sums can be computed by introducing additional symbols tuples::

    >>> from sympy import summation, oo, symbols, log
    >>> i, n, m = symbols('i n m', integer=True)

    >>> summation(2*i - 1, (i, 1, n))
    n**2
    >>> summation(1/2**i, (i, 0, oo))
    2
    >>> summation(1/log(n)**n, (n, 2, oo))
    Sum(log(n)**(-n), (n, 2, oo))
    >>> summation(i, (i, 0, n), (n, 0, m))
    m**3/6 + m**2/2 + m/3

    >>> from sympy.abc import x
    >>> from sympy import factorial
    >>> summation(x**n/factorial(n), (n, 0, oo))
    exp(x)

    """
    return Sum(f, *symbols, **kwargs).doit(deep=False)

def telescopic_direct(L, R, n, limits):
    """Returns the direct summation of the terms of a telescopic sum

    L is the term with lower index
    R is the term with higher index
    n difference between the indexes of L and R

    For example:

    >>> from sympy.concrete.summations import telescopic_direct
    >>> from sympy.abc import k, a, b
    >>> telescopic_direct(1/k, -1/(k+2), 2, (k, a, b))
    -1/(b + 2) - 1/(b + 1) + 1/(a + 1) + 1/a

    """
    (i, a, b) = limits
    s = 0
    for m in range(n):
        s += L.subs(i,a+m) + R.subs(i,b-m)
    return s

def telescopic(L, R, limits):
    '''Tries to perform the summation using the telescopic property

    return None if not possible
    '''
    (i, a, b) = limits
    if L.is_Add or R.is_Add:
        return None

    # We want to solve(L.subs(i, i + m) + R, m)
    # First we try a simple match since this does things that
    # solve doesn't do, e.g. solve(f(k+m)-f(k), m) fails

    k = Wild("k")
    sol = (-R).match(L.subs(i, i + k))
    s = None
    if sol and k in sol:
        s = sol[k]
        if not (s.is_Integer and L.subs(i,i + s) == -R):
            #sometimes match fail(f(x+2).match(-f(x+k))->{k: -2 - 2x}))
            s = None

    # But there are things that match doesn't do that solve
    # can do, e.g. determine that 1/(x + m) = 1/(1 - x) when m = 1

    if s is None:
        m = Dummy('m')
        try:
            sol = solve(L.subs(i, i + m) + R, m) or []
        except NotImplementedError:
            return None
        sol = [si for si in sol if si.is_Integer and
                                   (L.subs(i,i + si) + R).expand().is_zero]
        if len(sol) != 1:
            return None
        s = sol[0]

    if s < 0:
        return telescopic_direct(R, L, abs(s), (i, a, b))
    elif s > 0:
        return telescopic_direct(L, R, s, (i, a, b))

def eval_sum(f, limits):
    (i, a, b) = limits
    if f is S.Zero:
        return S.Zero
    if i not in f.free_symbols:
        return f*(b - a + 1)
    if a == b:
        return f.subs(i, a)

    dif = b - a
    definite = dif.is_Integer
    # Doing it directly may be faster if there are very few terms.
    if definite and (dif < 100):
        return eval_sum_direct(f, (i, a, b))
    # Try to do it symbolically. Even when the number of terms is known,
    # this can save time when b-a is big.
    # We should try to transform to partial fractions
    value = eval_sum_symbolic(f.expand(), (i, a, b))
    if value is not None:
        return value
    # Do it directly
    if definite:
        return eval_sum_direct(f, (i, a, b))

def eval_sum_direct(expr, limits):
    (i, a, b) = limits

    dif = b - a
    return Add(*[expr.subs(i, a + j) for j in range(dif + 1)])

def eval_sum_symbolic(f, limits):
    (i, a, b) = limits
    if not f.has(i):
        return f*(b-a+1)

    # Linearity
    if f.is_Mul:
        L, R = f.as_two_terms()

        if not L.has(i):
            sR = eval_sum_symbolic(R, (i, a, b))
            if sR: return L*sR

        if not R.has(i):
            sL = eval_sum_symbolic(L, (i, a, b))
            if sL: return R*sL

        try:
            f = apart(f, i) # see if it becomes an Add
        except PolynomialError:
            pass

    if f.is_Add:
        L, R = f.as_two_terms()
        lrsum = telescopic(L, R, (i, a, b))

        if lrsum:
            return lrsum

        lsum = eval_sum_symbolic(L, (i, a, b))
        rsum = eval_sum_symbolic(R, (i, a, b))

        if None not in (lsum, rsum):
            return lsum + rsum

    # Polynomial terms with Faulhaber's formula
    n = Wild('n')
    result = f.match(i**n)

    if result is not None:
        n = result[n]

        if n.is_Integer:
            if n >= 0:
                return ((C.bernoulli(n+1, b+1) - C.bernoulli(n+1, a))/(n+1)).expand()
            elif a.is_Integer and a >= 1:
                if n == -1:
                    return C.harmonic(b) - C.harmonic(a - 1)
                else:
                    return C.harmonic(b, abs(n)) - C.harmonic(a - 1, abs(n))

    # Geometric terms
    c1 = C.Wild('c1', exclude=[i])
    c2 = C.Wild('c2', exclude=[i])
    c3 = C.Wild('c3', exclude=[i])

    e = f.match(c1**(c2*i+c3))

    if e is not None:
        c1 = c1.subs(e)
        c2 = c2.subs(e)
        c3 = c3.subs(e)

        # TODO: more general limit handling
        return c1**c3 * (c1**(a*c2) - c1**(c2+b*c2)) / (1 - c1**c2)

    if not (a.has(S.Infinity, S.NegativeInfinity) or \
            b.has(S.Infinity, S.NegativeInfinity)):
        r = gosper_sum(f, (i, a, b))

        if not r in (None, S.NaN):
            return r

    return eval_sum_hyper(f, (i, a, b))

def _eval_sum_hyper(f, i, a):
    """ Returns (res, cond). Sums from a to oo. """
    from sympy.functions import hyper
    from sympy.simplify import hyperexpand, hypersimp, fraction, simplify
    from sympy.polys.polytools import Poly, factor

    if a != 0:
        return _eval_sum_hyper(f.subs(i, i + a), i, 0)

    if f.subs(i, 0) == 0:
        if simplify(f.subs(i, Dummy('i', integer=True, positive=True))) == 0:
            return S(0), True
        return _eval_sum_hyper(f.subs(i, i + 1), i, 0)

    hs = hypersimp(f, i)
    if hs is None:
        return None

    numer, denom = fraction(factor(hs))
    top, topl = numer.as_coeff_mul(i)
    bot, botl = denom.as_coeff_mul(i)
    ab = [top, bot]
    factors = [topl, botl]
    params = [[], []]
    for k in range(2):
        for fac in factors[k]:
            mul = 1
            if fac.is_Pow:
                mul = fac.exp
                fac = fac.base
                if not mul.is_Integer:
                    return None
            p = Poly(fac, i)
            if p.degree() != 1:
                return None
            m, n = p.all_coeffs()
            ab[k] *= m**mul
            params[k] += [n/m]*mul

    # Add "1" to numerator parameters, to account for implicit n! in
    # hypergeometric series.
    ap = params[0] + [1]
    bq = params[1]
    x  = ab[0]/ab[1]
    h = hyper(ap, bq, x)

    return f.subs(i, 0)*hyperexpand(h), h.convergence_statement

def eval_sum_hyper(f, xxx_todo_changeme):
    (i, a, b) = xxx_todo_changeme
    from sympy.functions import Piecewise
    from sympy import oo, And

    if b != oo:
        if a == -oo:
            res = _eval_sum_hyper(f.subs(i, -i), i, -b)
            if res is not None:
                return Piecewise(res, (Sum(f, (i, a, b)), True))
        else:
            res1 = _eval_sum_hyper(f, i, a)
            res2 = _eval_sum_hyper(f, i, b + 1)
            if res1 is None or res2 is None:
                return None
            (res1, cond1), (res2, cond2) = res1, res2
            cond = And(cond1, cond2)
            if cond is False:
                return None
        return Piecewise((res1 - res2, cond), (Sum(f, (i, a, b)), True))

    if a == -oo:
        res1 = _eval_sum_hyper(f.subs(i, -i), i, 1)
        res2 = _eval_sum_hyper(f, i, 0)
        if res1 is None or res2 is None:
            return None
        res1, cond1 = res1
        res2, cond2 = res2
        cond = And(cond1, cond2)
        if cond is False:
            return None
        return Piecewise((res1 + res2, cond), (Sum(f, (i, a, b)), True))

    # Now b == oo, a != -oo
    res = _eval_sum_hyper(f, i, a)
    if res is not None:
        return Piecewise(res, (Sum(f, (i, a, b)), True))




from sympy import S, sqrt, Sum, symbols
from sympy.physics.quantum.cg import Wigner3j, Wigner6j, Wigner9j, CG, cg_simp
from sympy.functions.special.tensor_functions import KroneckerDelta


def test_cg_simp_add():
    j, m1, m1p, m2, m2p = symbols('j m1 m1p m2 m2p')
    # Test Varshalovich 8.7.1 Eq 1
    a = CG(S(1)/2,S(1)/2,0,0,S(1)/2,S(1)/2)
    b = CG(S(1)/2,-S(1)/2,0,0,S(1)/2,-S(1)/2)
    c = CG(1,1,0,0,1,1)
    d = CG(1,0,0,0,1,0)
    e = CG(1,-1,0,0,1,-1)
    assert cg_simp(a+b) == 2
    assert cg_simp(c+d+e) == 3
    assert cg_simp(a+b+c+d+e) == 5
    assert cg_simp(a+b+c) == 2+c
    assert cg_simp(2*a+b) == 2+a
    assert cg_simp(2*c+d+e) == 3+c
    assert cg_simp(5*a+5*b) == 10
    assert cg_simp(5*c+5*d+5*e) == 15
    assert cg_simp(-a-b) == -2
    assert cg_simp(-c-d-e) == -3
    assert cg_simp(-6*a-6*b) == -12
    assert cg_simp(-4*c-4*d-4*e) == -12
    a = CG(S(1)/2,S(1)/2,j,0,S(1)/2,S(1)/2)
    b = CG(S(1)/2,-S(1)/2,j,0,S(1)/2,-S(1)/2)
    c = CG(1,1,j,0,1,1)
    d = CG(1,0,j,0,1,0)
    e = CG(1,-1,j,0,1,-1)
    assert cg_simp(a+b) == 2*KroneckerDelta(j,0)
    assert cg_simp(c+d+e) == 3*KroneckerDelta(j,0)
    assert cg_simp(a+b+c+d+e) == 5*KroneckerDelta(j,0)
    assert cg_simp(a+b+c) == 2*KroneckerDelta(j,0)+c
    assert cg_simp(2*a+b) == 2*KroneckerDelta(j,0)+a
    assert cg_simp(2*c+d+e) == 3*KroneckerDelta(j,0)+c
    assert cg_simp(5*a+5*b) == 10*KroneckerDelta(j,0)
    assert cg_simp(5*c+5*d+5*e) == 15*KroneckerDelta(j,0)
    assert cg_simp(-a-b) == -2*KroneckerDelta(j,0)
    assert cg_simp(-c-d-e) == -3*KroneckerDelta(j,0)
    assert cg_simp(-6*a-6*b) == -12*KroneckerDelta(j,0)
    assert cg_simp(-4*c-4*d-4*e) == -12*KroneckerDelta(j,0)
    # Test Varshalovich 8.7.1 Eq 2
    a = CG(S(1)/2,S(1)/2,S(1)/2,-S(1)/2,0,0)
    b = CG(S(1)/2,-S(1)/2,S(1)/2,S(1)/2,0,0)
    c = CG(1,1,1,-1,0,0)
    d = CG(1,0,1,0,0,0)
    e = CG(1,-1,1,1,0,0)
    assert cg_simp(a-b) == sqrt(2)
    assert cg_simp(c-d+e) == sqrt(3)
    assert cg_simp(a-b+c-d+e) == sqrt(2)+sqrt(3)
    assert cg_simp(a-b+c) == sqrt(2)+c
    assert cg_simp(2*a-b) == sqrt(2)+a
    assert cg_simp(2*c-d+e) == sqrt(3)+c
    assert cg_simp(5*a-5*b) == 5*sqrt(2)
    assert cg_simp(5*c-5*d+5*e) == 5*sqrt(3)
    assert cg_simp(-a+b) == -sqrt(2)
    assert cg_simp(-c+d-e) == -sqrt(3)
    assert cg_simp(-6*a+6*b) == -6*sqrt(2)
    assert cg_simp(-4*c+4*d-4*e) == -4*sqrt(3)
    a = CG(S(1)/2,S(1)/2,S(1)/2,-S(1)/2,j,0)
    b = CG(S(1)/2,-S(1)/2,S(1)/2,S(1)/2,j,0)
    c = CG(1,1,1,-1,j,0)
    d = CG(1,0,1,0,j,0)
    e = CG(1,-1,1,1,j,0)
    assert cg_simp(a-b) == sqrt(2)*KroneckerDelta(j,0)
    assert cg_simp(c-d+e) == sqrt(3)*KroneckerDelta(j,0)
    assert cg_simp(a-b+c-d+e) == sqrt(2)*KroneckerDelta(j,0)+sqrt(3)*KroneckerDelta(j,0)
    assert cg_simp(a-b+c) == sqrt(2)*KroneckerDelta(j,0)+c
    assert cg_simp(2*a-b) == sqrt(2)*KroneckerDelta(j,0)+a
    assert cg_simp(2*c-d+e) == sqrt(3)*KroneckerDelta(j,0)+c
    assert cg_simp(5*a-5*b) == 5*sqrt(2)*KroneckerDelta(j,0)
    assert cg_simp(5*c-5*d+5*e) == 5*sqrt(3)*KroneckerDelta(j,0)
    assert cg_simp(-a+b) == -sqrt(2)*KroneckerDelta(j,0)
    assert cg_simp(-c+d-e) == -sqrt(3)*KroneckerDelta(j,0)
    assert cg_simp(-6*a+6*b) == -6*sqrt(2)*KroneckerDelta(j,0)
    assert cg_simp(-4*c+4*d-4*e) == -4*sqrt(3)*KroneckerDelta(j,0)
    # Test Varshalovich 8.7.2 Eq 9
    # alpha=alphap,beta=betap case
    # numerical
    a = CG(S(1)/2,S(1)/2,S(1)/2,-S(1)/2,1,0)**2
    b = CG(S(1)/2,S(1)/2,S(1)/2,-S(1)/2,0,0)**2
    c = CG(1,0,1,1,1,1)**2
    d = CG(1,0,1,1,2,1)**2
    assert cg_simp(a+b) == 1
    assert cg_simp(c+d) == 1
    assert cg_simp(a+b+c+d) == 2
    assert cg_simp(4*a+4*b) == 4
    assert cg_simp(4*c+4*d) == 4
    assert cg_simp(5*a+3*b) == 3+2*a
    assert cg_simp(5*c+3*d) == 3+2*c
    assert cg_simp(-a-b) == -1
    assert cg_simp(-c-d) == -1
    # symbolic
    a = CG(S(1)/2,m1,S(1)/2,m2,1,1)**2
    b = CG(S(1)/2,m1,S(1)/2,m2,1,0)**2
    c = CG(S(1)/2,m1,S(1)/2,m2,1,-1)**2
    d = CG(S(1)/2,m1,S(1)/2,m2,0,0)**2
    assert cg_simp(a+b+c+d) == 1
    assert cg_simp(4*a+4*b+4*c+4*d) == 4
    assert cg_simp(3*a+5*b+3*c+4*d) == 3+2*b+d
    assert cg_simp(-a-b-c-d) == -1
    a = CG(1,m1,1,m2,2,2)**2
    b = CG(1,m1,1,m2,2,1)**2
    c = CG(1,m1,1,m2,2,0)**2
    d = CG(1,m1,1,m2,2,-1)**2
    e = CG(1,m1,1,m2,2,-2)**2
    f = CG(1,m1,1,m2,1,1)**2
    g = CG(1,m1,1,m2,1,0)**2
    h = CG(1,m1,1,m2,1,-1)**2
    i = CG(1,m1,1,m2,0,0)**2
    assert cg_simp(a+b+c+d+e+f+g+h+i) == 1
    assert cg_simp(4*(a+b+c+d+e+f+g+h+i)) == 4
    assert cg_simp(a+b+2*c+d+4*e+f+g+h+i) == 1+c+3*e
    assert cg_simp(-a-b-c-d-e-f-g-h-i) == -1
    # alpha!=alphap or beta!=betap case
    # numerical
    a = CG(S(1)/2,S(1)/2,S(1)/2,-S(1)/2,1,0)*CG(S(1)/2,-S(1)/2,S(1)/2,S(1)/2,1,0)
    b = CG(S(1)/2,S(1)/2,S(1)/2,-S(1)/2,0,0)*CG(S(1)/2,-S(1)/2,S(1)/2,S(1)/2,0,0)
    c = CG(1,1,1,0,2,1)*CG(1,0,1,1,2,1)
    d = CG(1,1,1,0,1,1)*CG(1,0,1,1,1,1)
    assert cg_simp(a+b) == 0
    assert cg_simp(c+d) == 0
    # symbolic
    a = CG(S(1)/2,m1,S(1)/2,m2,1,1)*CG(S(1)/2,m1p,S(1)/2,m2p,1,1)
    b = CG(S(1)/2,m1,S(1)/2,m2,1,0)*CG(S(1)/2,m1p,S(1)/2,m2p,1,0)
    c = CG(S(1)/2,m1,S(1)/2,m2,1,-1)*CG(S(1)/2,m1p,S(1)/2,m2p,1,-1)
    d = CG(S(1)/2,m1,S(1)/2,m2,0,0)*CG(S(1)/2,m1p,S(1)/2,m2p,0,0)
    assert cg_simp(a+b+c+d) == KroneckerDelta(m1,m1p)*KroneckerDelta(m2,m2p)
    a = CG(1,m1,1,m2,2,2)*CG(1,m1p,1,m2p,2,2)
    b = CG(1,m1,1,m2,2,1)*CG(1,m1p,1,m2p,2,1)
    c = CG(1,m1,1,m2,2,0)*CG(1,m1p,1,m2p,2,0)
    d = CG(1,m1,1,m2,2,-1)*CG(1,m1p,1,m2p,2,-1)
    e = CG(1,m1,1,m2,2,-2)*CG(1,m1p,1,m2p,2,-2)
    f = CG(1,m1,1,m2,1,1)*CG(1,m1p,1,m2p,1,1)
    g = CG(1,m1,1,m2,1,0)*CG(1,m1p,1,m2p,1,0)
    h = CG(1,m1,1,m2,1,-1)*CG(1,m1p,1,m2p,1,-1)
    i = CG(1,m1,1,m2,0,0)*CG(1,m1p,1,m2p,0,0)
    assert cg_simp(a+b+c+d+e+f+g+h+i) == KroneckerDelta(m1,m1p)*KroneckerDelta(m2,m2p)

def test_cg_simp_sum():
    x, a, b, c, cp, alpha, beta, gamma, gammap = symbols('x a b c cp alpha beta gamma gammap')
    # Varshalovich 8.7.1 Eq 1
    assert cg_simp(x * Sum(CG(a,alpha,b,0,a,alpha), (alpha,-a,a))) == x*(2*a+1)*KroneckerDelta(b,0)
    assert cg_simp(x * Sum(CG(a,alpha,b,0,a,alpha), (alpha,-a,a))+CG(1,0,1,0,1,0)) == x*(2*a+1)*KroneckerDelta(b,0)+CG(1,0,1,0,1,0)
    assert cg_simp(2 * Sum(CG(1,alpha,0,0,1,alpha), (alpha,-1,1))) == 6
    # Varshalovich 8.7.1 Eq 2
    assert cg_simp(x*Sum((-1)**(a-alpha) * CG(a,alpha,a,-alpha,c,0), (alpha,-a,a))) == x*sqrt(2*a+1)*KroneckerDelta(c,0)
    assert cg_simp(3*Sum((-1)**(2-alpha) * CG(2,alpha,2,-alpha,0,0), (alpha,-2,2))) == 3*sqrt(5)
    # Varshalovich 8.7.2 Eq 4
    assert cg_simp(Sum(CG(a,alpha,b,beta,c,gamma)*CG(a,alpha,b,beta,cp,gammap),(alpha,-a,a),(beta,-b,b))) == KroneckerDelta(c,cp)*KroneckerDelta(gamma,gammap)
    assert cg_simp(Sum(CG(a,alpha,b,beta,c,gamma)*CG(a,alpha,b,beta,c,gammap),(alpha,-a,a),(beta,-b,b))) == KroneckerDelta(gamma,gammap)
    assert cg_simp(Sum(CG(a,alpha,b,beta,c,gamma)*CG(a,alpha,b,beta,cp,gamma),(alpha,-a,a),(beta,-b,b))) == KroneckerDelta(c,cp)
    assert cg_simp(Sum(CG(a,alpha,b,beta,c,gamma)**2,(alpha,-a,a),(beta,-b,b))) == 1
    assert cg_simp(Sum(CG(2,alpha,1,beta,2,gamma)*CG(2,alpha,1,beta,2,gammap), (alpha,-2,2), (beta,-1,1))) == KroneckerDelta(gamma,gammap)

def test_doit():
    assert Wigner3j(1/2,-1/2,1/2,1/2,0,0).doit() == -sqrt(2)/2
    assert Wigner6j(1,2,3,2,1,2).doit() == sqrt(21)/105
    assert Wigner9j(2,1,1,S(3)/2,S(1)/2,1,S(1)/2,S(1)/2,0).doit() == sqrt(2)/12
    assert CG(1/2,1/2,1/2,-1/2,1,0).doit() == sqrt(2)/2



"""Printing subsystem"""

from .pretty import *
from .latex import latex, print_latex
from .mathml import mathml, print_mathml
from .python import python, print_python
from .ccode import ccode, print_ccode
from .fcode import fcode, print_fcode
from .jscode import jscode, print_jscode
from .gtk import *
from .preview import preview
from .repr import srepr
from .tree import print_tree
from .str import StrPrinter, sstr, sstrrepr
from .tableform import TableForm



"""Sparse distributed multivariate polynomials. """

from sympy.polys.monomialtools import (
    monomial_mul, monomial_div, monomial_lcm, lex,
)

from sympy.polys.polyerrors import (
    ExactQuotientFailed,
)

def sdp_LC(f, K):
    """Returns the leading coeffcient of `f`. """
    if not f:
        return K.zero
    else:
        return f[0][1]

def sdp_LM(f, u):
    """Returns the leading monomial of `f`. """
    if not f:
        return (0,) * (u + 1)
    else:
        return f[0][0]

def sdp_LT(f, u, K):
    """Returns the leading term of `f`. """
    if f:
        return f[0]
    else:
        return (0,) * (u + 1), K.zero

def sdp_del_LT(f):
    """Removes the leading from `f`. """
    return f[1:]

def sdp_coeffs(f):
    """Returns a list of monomials in `f`. """
    return [ coeff for _, coeff in f ]

def sdp_monoms(f):
    """Returns a list of monomials in `f`. """
    return [ monom for monom, _ in f ]

def sdp_sort(f, O):
    """Sort terms in `f` using the given monomial order `O`. """
    return sorted(f, key=lambda term: O(term[0]), reverse=True)

def sdp_strip(f):
    """Remove terms with zero coefficients from `f` in `K[X]`. """
    return [ (monom, coeff) for monom, coeff in f if coeff ]

def sdp_normal(f, K):
    """Normalize distributed polynomial in the given domain. """
    return [ (monom, K.convert(coeff)) for monom, coeff in f if coeff ]

def sdp_from_dict(f, O):
    """Make a distributed polynomial from a dictionary. """
    return sdp_sort(list(f.items()), O)

def sdp_to_dict(f):
    """Make a dictionary from a distributed polynomial. """
    return dict(f)

def sdp_indep_p(f, j, u):
    """Returns `True` if a polynomial is independent of `x_j`. """
    if j < 0 or j > u:
        raise IndexError("-%s <= j < %s expected, got %s" % (u, u, j))
    else:
        return all(not monom[j] for monom in sdp_monoms(h))

def sdp_one_p(f, u, K):
    """Returns True if `f` is a multivariate one in `K[X]`. """
    return f == sdp_one(u, K)

def sdp_one(u, K):
    """Returns a multivariate one in `K[X]`. """
    return (((0,) * (u + 1), K.one),)

def sdp_term_p(f):
    """Returns True if `f` has a single term or is zero. """
    return len(f) <= 1

def sdp_abs(f, u, O, K):
    """Make all coefficients positive in `K[X]`. """
    return [ (monom, K.abs(coeff)) for monom, coeff in f ]

def sdp_neg(f, u, O, K):
    """Negate a polynomial in `K[X]`. """
    return [ (monom, -coeff) for monom, coeff in f ]

def sdp_add_term(f, term, u, O, K):
    """Add a single term using bisection method. """
    M, c = term

    if not c:
        return f
    if not f:
        return [(M, c)]

    monoms = sdp_monoms(f)

    if O(M) > O(monoms[ 0]):
        return [(M, c)] + f
    if O(M) < O(monoms[-1]):
        return f + [(M, c)]

    lo, hi = 0, len(monoms) - 1

    while lo <= hi:
        i = (lo + hi) // 2

        if O(M) == O(monoms[i]):
            coeff = f[i][1] + c

            if not coeff:
                return f[:i] + f[i + 1:]
            else:
                return f[:i] + [(M, coeff)] + f[i + 1:]
        else:
            if O(M) > O(monoms[i]):
                hi = i - 1
            else:
                lo = i + 1
    else:
        return f[:i] + [(M, c)] + f[i + 1:]

def sdp_sub_term(f, term, u, O, K):
    """Sub a single term using bisection method. """
    M, c = term

    if not c:
        return f
    if not f:
        return [(M, -c)]

    monoms = sdp_monoms(f)

    if O(M) > O(monoms[ 0]):
        return [(M, -c)] + f
    if O(M) < O(monoms[-1]):
        return f + [(M, -c)]

    lo, hi = 0, len(monoms) - 1

    while lo <= hi:
        i = (lo + hi) // 2

        if O(M) == O(monoms[i]):
            coeff = f[i][1] - c

            if not coeff:
                return f[:i] + f[i + 1:]
            else:
                return f[:i] + [(M, coeff)] + f[i + 1:]
        else:
            if O(M) > O(monoms[i]):
                hi = i - 1
            else:
                lo = i + 1
    else:
        return f[:i] + [(M, -c)] + f[i + 1:]

def sdp_mul_term(f, term, u, O, K):
    """Multiply a distributed polynomial by a term. """
    M, c = term

    if not f or not c:
        return []
    else:
        if K.is_one(c):
            return [ (monomial_mul(f_M, M), f_c) for f_M, f_c in f ]
        else:
            return [ (monomial_mul(f_M, M), f_c * c) for f_M, f_c in f ]

def sdp_add(f, g, u, O, K):
    """Add distributed polynomials in `K[X]`. """
    h = dict(f)

    for monom, c in g:
        if monom in h:
            coeff = h[monom] + c

            if not coeff:
                del h[monom]
            else:
                h[monom] = coeff
        else:
            h[monom] = c

    return sdp_from_dict(h, O)

def sdp_sub(f, g, u, O, K):
    """Subtract distributed polynomials in `K[X]`. """
    h = dict(f)

    for monom, c in g:
        if monom in h:
            coeff = h[monom] - c

            if not coeff:
                del h[monom]
            else:
                h[monom] = coeff
        else:
            h[monom] = -c

    return sdp_from_dict(h, O)

def sdp_mul(f, g, u, O, K):
    """Multiply distributed polynomials in `K[X]`. """
    if sdp_term_p(f):
        if not f:
            return f
        else:
            return sdp_mul_term(g, f[0], u, O, K)

    if sdp_term_p(g):
        if not g:
            return g
        else:
            return sdp_mul_term(f, g[0], u, O, K)

    h = {}

    for fm, fc in f:
        for gm, gc in g:
            monom = monomial_mul(fm, gm)
            coeff = fc * gc

            if monom in h:
                coeff += h[monom]

                if not coeff:
                    del h[monom]
                    continue

            h[monom] = coeff

    return sdp_from_dict(h, O)

def sdp_sqr(f, u, O, K):
    """Square a distributed polynomial in `K[X]`. """
    h = {}

    for fm, fc in f:
        for Fm, Fc in f:
            monom = monomial_mul(fm, Fm)
            coeff = fc * Fc

            if monom in h:
                coeff += h[monom]

                if not coeff:
                    del h[monom]
                    continue

            h[monom] = coeff

    return sdp_from_dict(h, O)

def sdp_pow(f, n, u, O, K):
    """Raise `f` to the n-th power in `K[X]`. """
    if not n:
        return sdp_one(u, K)
    if n < 0:
        raise ValueError("can't raise a polynomial to negative power")
    if n == 1 or not f or sdp_one_p(f, u, K):
        return f

    g = sdp_one(u, K)

    while True:
        n, m = n // 2, n

        if m & 1:
            g = sdp_mul(g, f, u, O, K)

            if not n:
                break

        f = sdp_sqr(f, u, O, K)

    return g

def sdp_monic(f, K):
    """Divides all coefficients by `LC(f)` in `K[X]`. """
    if not f:
        return f

    lc_f = sdp_LC(f, K)

    if K.is_one(lc_f):
        return f
    else:
        return [ (m, K.quo(c, lc_f)) for m, c in f ]

def sdp_content(f, K):
    """Returns GCD of coefficients in `K[X]`. """
    from sympy.polys.domains import QQ

    if K.has_Field:
        return K.one
    else:
        cont = K.zero

        if K == QQ:
            for c in f:
                cont = K.gcd(cont, c)
        else:
            for c in f:
                cont = K.gcd(cont, c)

                if K.is_one(cont) and i:
                    break

        return cont

def sdp_primitive(f, K):
    """Returns content and a primitive polynomial in `K[X]`. """
    if K.has_Field:
        return K.one, f
    else:
        cont = sdp_content(f, K)

        if K.is_one(cont):
            return cont, f
        else:
            return cont, [ (m, K.quo(c, cont)) for m, c in f ]

def _term_rr_div(a, b, K):
    """Division of two terms in over a ring. """
    a_lm, a_lc = a
    b_lm, b_lc = b

    monom = monomial_div(a_lm, b_lm)

    if not (monom is None or a_lc % b_lc):
        return monom, K.quo(a_lc, b_lc)
    else:
        return None

def _term_ff_div(a, b, K):
    """Division of two terms in over a field. """
    a_lm, a_lc = a
    b_lm, b_lc = b

    monom = monomial_div(a_lm, b_lm)

    if monom is not None:
        return monom, K.quo(a_lc, b_lc)
    else:
        return None

def sdp_div(f, G, u, O, K):
    """
    Generalized polynomial division with remainder in `K[X]`.

    Given polynomial `f` and a set of polynomials `g = (g_1, ..., g_n)`
    compute a set of quotients `q = (q_1, ..., q_n)` and remainder `r`
    such that `f = q_1*f_1 + ... + q_n*f_n + r`, where `r = 0` or `r`
    is a completely reduced polynomial with respect to `g`.

    References
    ==========

    1. [Cox97]_
    2. [Ajwa95]_

    """
    Q, r = [ [] for _ in range(len(G)) ], []

    if K.has_Field:
        term_div = _term_ff_div
    else:
        term_div = _term_rr_div

    while f:
        for i, g in enumerate(G):
            tq = term_div(sdp_LT(f, u, K), sdp_LT(g, u, K), K)

            if tq is not None:
                Q[i] = sdp_add_term(Q[i], tq, u, O, K)
                f = sdp_sub(f, sdp_mul_term(g, tq, u, O, K), u, O, K)

                break
        else:
            r = sdp_add_term(r, sdp_LT(f, u, K), u, O, K)
            f = sdp_del_LT(f)

    return Q, r



def sdp_rem(f, G, u, O, K):
    """Returns polynomial remainder in `K[X]`. """
    r = {}

    if K.has_Field:
        term_div = _term_ff_div
    else:
        term_div = _term_rr_div

    ltf = sdp_LT(f, u, K)
    f = dict(f)
    get = f.get
    while f:
        for g in G:
            tq = term_div(ltf, sdp_LT(g, u, K), K)

            if tq is not None:
                m, c = tq
                for mg, cg in g:
                    m1 = monomial_mul(mg, m)
                    c1 = get(m1, 0) - c*cg
                    if not c1:
                        del f[m1]
                    else:
                        f[m1] = c1
                if f:
                    if O == lex:
                        ltm = max(f)
                    else:
                        ltm = max(f, key=lambda mx: O(mx))
                    ltf = ltm, f[ltm]

                break
        else:
            ltm, ltc = ltf
            if ltm in r:
                r[ltm] += ltc
            else:
                r[ltm] = ltc
            del f[ltm]
            if f:
                if O == lex:
                    ltm = max(f)
                else:
                    ltm = max(f, key=lambda mx: O(mx))
                ltf = ltm, f[ltm]
    return sdp_from_dict(r, O)


def sdp_quo(f, g, u, O, K):
    """Returns polynomial quotient in `K[x]`. """
    return sdp_div(f, g, u, O, K)[0]

def sdp_exquo(f, g, u, O, K):
    """Returns exact polynomial quotient in `K[X]`. """
    q, r = sdp_div(f, g, u, O, K)

    if not r:
        return q
    else:
        raise ExactQuotientFailed(f, g)

def sdp_lcm(f, g, u, O, K):
    """
    Computes LCM of two polynomials in `K[X]`.

    The LCM is computed as the unique generater of the intersection
    of the two ideals generated by `f` and `g`. The approach is to
    compute a Groebner basis with respect to lexicographic ordering
    of `t*f` and `(1 - t)*g`, where `t` is an unrelated variable and
    then filtering out the solution that doesn't contain `t`.

    References
    ==========

    1. [Cox97]_

    """
    from sympy.polys.groebnertools import sdp_groebner

    if not f or not g:
        return []

    if sdp_term_p(f) and sdp_term_p(g):
        monom = monomial_lcm(sdp_LM(f, u), sdp_LM(g, u))

        fc, gc = sdp_LC(f, K), sdp_LC(g, K)

        if K.has_Field:
            coeff = K.one
        else:
            coeff = K.lcm(fc, gc)

        return [(monom, coeff)]

    if not K.has_Field:
        lcm = K.one
    else:
        fc, f = sdp_primitive(f, K)
        gc, g = sdp_primitive(g, K)

        lcm = K.lcm(fc, gc)

    f_terms = tuple( ((1,) + m,  c) for m, c in f )
    g_terms = tuple( ((0,) + m,  c) for m, c in g ) \
            + tuple( ((1,) + m, -c) for m, c in g )

    F = sdp_sort(f_terms, lex)
    G = sdp_sort(g_terms, lex)

    basis = sdp_groebner([F, G], u, lex, K)

    H = [ h for h in basis if sdp_indep_p(h, 0, u) ]

    if K.is_one(lcm):
        h = [ (m[1:], c)     for m, c in H[0] ]
    else:
        h = [ (m[1:], c * lcm) for m, c in H[0] ]

    return sdp_sort(h, O)

def sdp_gcd(f, g, u, O, K):
    """Compute GCD of two polynomials in `K[X]` via LCM. """
    if not K.has_Field:
        fc, f = sdp_primitive(f, K)
        gc, g = sdp_primitive(g, K)

        gcd = K.gcd(fc, gc)

    h = sdp_quo(sdp_mul(f, g, u, O, K),
                sdp_lcm(f, g, u, O, K), u, O, K)

    if not K.has_Field:
        if K.is_one(gcd):
            return h
        else:
            return [ (m, c * gcd) for m, c in h ]
    else:
        return sdp_monic(h, K)



from sympy import symbols, Symbol, Eijk, LeviCivita, KroneckerDelta, Dummy
from sympy.physics.secondquant import evaluate_deltas, F
from sympy.utilities.pytest import XFAIL

x, y = symbols('x,y')

def test_levicivita():
    assert Eijk(1, 2, 3) == LeviCivita(1, 2, 3)
    assert LeviCivita(1, 2, 3) == 1
    assert LeviCivita(1, 3, 2) == -1
    assert LeviCivita(1, 2, 2) == 0
    i,j,k = symbols('i j k')
    assert LeviCivita(i, j, k) == LeviCivita(i,j,k, evaluate=False)
    assert LeviCivita(i, j, i) == 0
    assert LeviCivita(1, i, i) == 0
    assert LeviCivita(i, j, k).doit() == (j - i)*(k - i)*(k - j)/2
    assert LeviCivita(1, 2, 3, 1) == 0
    assert LeviCivita(4, 5, 1, 2, 3) == 1
    assert LeviCivita(4, 5, 2, 1, 3) == -1

def test_kronecker_delta():
    i, j = symbols('i,j')
    k = Symbol('k', nonzero=True)
    assert KroneckerDelta(1, 1) == 1
    assert KroneckerDelta(1, 2) == 0
    assert KroneckerDelta(x, x) == 1
    assert KroneckerDelta(x**2-y**2, x**2-y**2) == 1
    assert KroneckerDelta(i, i) == 1
    assert KroneckerDelta(i, i + 1) == 0
    assert KroneckerDelta(0, 0) == 1
    assert KroneckerDelta(0, 1) == 0
    assert KroneckerDelta(i + k, i + k) == 1
    assert KroneckerDelta(i + k, i + 1 + k) == 0
    assert KroneckerDelta(i, j).subs(dict(i=1, j=0)) == 0
    assert KroneckerDelta(i, j).subs(dict(i=3, j=3)) == 1


def test_kronecker_delta_secondquant():
    """secondquant-specific methods"""
    D = KroneckerDelta
    i,j,v,w = symbols('i j v w',below_fermi=True,cls=Dummy)
    a,b,t,u = symbols('a b t u',above_fermi=True, cls=Dummy)
    p,q,r,s = symbols('p q r s',cls=Dummy)

    assert D(i,a) == 0
    assert D(i,t) == 0

    assert D(i,j).is_above_fermi == False
    assert D(a,b).is_above_fermi == True
    assert D(p,q).is_above_fermi == True
    assert D(i,q).is_above_fermi == False
    assert D(q,i).is_above_fermi == False
    assert D(q,v).is_above_fermi == False
    assert D(a,q).is_above_fermi == True

    assert D(i,j).is_below_fermi == True
    assert D(a,b).is_below_fermi == False
    assert D(p,q).is_below_fermi == True
    assert D(p,j).is_below_fermi == True
    assert D(q,b).is_below_fermi == False

    assert D(i,j).is_only_above_fermi == False
    assert D(a,b).is_only_above_fermi == True
    assert D(p,q).is_only_above_fermi == False
    assert D(i,q).is_only_above_fermi == False
    assert D(q,i).is_only_above_fermi == False
    assert D(a,q).is_only_above_fermi == True

    assert D(i,j).is_only_below_fermi == True
    assert D(a,b).is_only_below_fermi == False
    assert D(p,q).is_only_below_fermi == False
    assert D(p,j).is_only_below_fermi == True
    assert D(q,b).is_only_below_fermi == False

    assert not D(i,q).indices_contain_equal_information
    assert not D(a,q).indices_contain_equal_information
    assert D(p,q).indices_contain_equal_information
    assert D(a,b).indices_contain_equal_information
    assert D(i,j).indices_contain_equal_information

    assert D(q,b).preferred_index == b
    assert D(q,b).killable_index == q
    assert D(q,t).preferred_index == t
    assert D(q,t).killable_index == q
    assert D(q,i).preferred_index == i
    assert D(q,i).killable_index == q
    assert D(q,v).preferred_index == v
    assert D(q,v).killable_index == q
    assert D(q, p).preferred_index == q
    assert D(q, p).killable_index == p


    EV = evaluate_deltas
    assert EV(D(a,q)*F(q)) == F(a)
    assert EV(D(i,q)*F(q)) == F(i)
    assert EV(D(a,q)*F(a)) == D(a,q)*F(a)
    assert EV(D(i,q)*F(i)) == D(i,q)*F(i)
    assert EV(D(a,b)*F(a)) == F(b)
    assert EV(D(a,b)*F(b)) == F(a)
    assert EV(D(i,j)*F(i)) == F(j)
    assert EV(D(i,j)*F(j)) == F(i)
    assert EV(D(p,q)*F(q)) == F(p)
    assert EV(D(p,q)*F(p)) == F(q)
    assert EV(D(p,j)*D(p,i)*F(i)) == F(j)
    assert EV(D(p,j)*D(p,i)*F(j)) == F(i)
    assert EV(D(p,q)*D(p,i))*F(i) == D(q,i)*F(i)

@XFAIL
def test_kronecker_delta_failing():
    assert D(i, i + k) == D(0, k)



from sympy.utilities.pytest import raises
from sympy import Symbol, symbols, Function, Integer, Matrix, nan, oo, Abs, \
    Rational, Float, S, WildFunction, ImmutableMatrix
from sympy.geometry import Point, Circle, Ellipse
from sympy.printing import srepr

x, y = symbols('x,y')

# eval(srepr(expr)) == expr has to succeed in the right environment. The right
# environment is the scope of "from sympy import *" for most cases.
ENV = {}
exec("from sympy import *", ENV)

def sT(expr, string):
    """
    sT := sreprTest

    Tests that srepr delivers the expected string and that
    the condition eval(srepr(expr))==expr holds.
    """
    assert srepr(expr) == string
    assert eval(string, ENV) == expr

def test_printmethod():
    class R(Abs):
        def _sympyrepr(self, printer):
            return "foo(%s)" % printer._print(self.args[0])
    assert srepr(R(x)) == "foo(Symbol('x'))"

def test_Add():
    sT(x+y, "Add(Symbol('x'), Symbol('y'))")
    assert srepr(x**2 + 1, order='lex') == "Add(Pow(Symbol('x'), Integer(2)), Integer(1))"
    assert srepr(x**2 + 1, order='old') == "Add(Integer(1), Pow(Symbol('x'), Integer(2)))"

def test_Function():
    sT(Function("f")(x), "Function('f')(Symbol('x'))")
    # test unapplied Function
    sT(Function('f'), "Function('f')")

def test_Geometry():
    sT(Point(0,0),  "Point(Integer(0), Integer(0))")
    sT(Ellipse(Point(0, 0), 5, 1),  "Ellipse(Point(Integer(0), Integer(0)), Integer(5), Integer(1))")
    # TODO more tests

def test_Singletons():
    sT(S.Catalan, 'Catalan')
    sT(S.ComplexInfinity, 'zoo')
    sT(S.EulerGamma, 'EulerGamma')
    sT(S.Exp1, 'E')
    sT(S.GoldenRatio, 'GoldenRatio')
    sT(S.Half, 'Rational(1, 2)')
    sT(S.ImaginaryUnit, 'I')
    sT(S.Infinity, 'oo')
    sT(S.NaN, 'nan')
    sT(S.NegativeInfinity, '-oo')
    sT(S.NegativeOne, 'Integer(-1)')
    sT(S.One, 'Integer(1)')
    sT(S.Pi, 'pi')
    sT(S.Zero, 'Integer(0)')

def test_Integer():
    sT(Integer(4), "Integer(4)")

def test_list():
    sT([x, Integer(4)], "[Symbol('x'), Integer(4)]")

def test_Matrix():
    # Matrix is really MutableMatrix
    for cls, name in [(Matrix, "Matrix"), (ImmutableMatrix, "ImmutableMatrix")]:
        sT(cls([[x**+1, 1], [y, x+y]]),
           "%s([[Symbol('x'), Integer(1)], [Symbol('y'), Add(Symbol('x'), Symbol('y'))]])"%name)

        sT(cls(), "%s([])"%name)

        sT(cls([[x**+1, 1], [y, x+y]]), "%s([[Symbol('x'), Integer(1)], [Symbol('y'), Add(Symbol('x'), Symbol('y'))]])"%name)

def test_Rational():
    sT(Rational(1,3), "Rational(1, 3)")
    sT(Rational(-1,3), "Rational(-1, 3)")

def test_Float():
    sT(Float('1.23', prec=3), "Float('1.22998', prec=3)")
    sT(Float('1.23456789', prec=9), "Float('1.23456788994', prec=9)")
    sT(Float('1.234567890123456789', prec=19), "Float('1.234567890123456789013', prec=19)")
    sT(Float('0.60038617995049726', 15), "Float('0.60038617995049726', prec=15)")

def test_Symbol():
    sT(x, "Symbol('x')")
    sT(y, "Symbol('y')")

def test_tuple():
    sT((x,), "(Symbol('x'),)")
    sT((x,y), "(Symbol('x'), Symbol('y'))")

def test_WildFunction():
    sT(WildFunction('w'), "WildFunction('w')")

def test_settins():
    raises(TypeError, lambda: srepr(x, method="garbage"))

def test_Mul():
    sT(3*x**3*y, "Mul(Integer(3), Pow(Symbol('x'), Integer(3)), Symbol('y'))")
    assert srepr(3*x**3*y, order='old') == "Mul(Integer(3), Symbol('y'), Pow(Symbol('x'), Integer(3)))"



"""sympify -- convert objects SymPy internal format"""

from inspect import getmro

from .core import all_classes as sympy_classes
from sympy.core.compatibility import iterable


class SympifyError(ValueError):
    def __init__(self, expr, base_exc=None):
        self.expr = expr
        self.base_exc = base_exc
    def __str__(self):
        if self.base_exc is None:
            return "SympifyError: %r" % (self.expr,)

        return ("Sympify of expression '%s' failed, because of exception being "
            "raised:\n%s: %s" % (self.expr, self.base_exc.__class__.__name__,
            str(self.base_exc)))

converter = {} #See sympify docstring.

def sympify(a, locals=None, convert_xor=True, strict=False, rational=False):
    """
    Converts an arbitrary expression to a type that can be used inside sympy.

    For example, it will convert python ints into instance of sympy.Rational,
    floats into instances of sympy.Float, etc. It is also able to coerce symbolic
    expressions which inherit from Basic. This can be useful in cooperation
    with SAGE.

    It currently accepts as arguments:
       - any object defined in sympy (except matrices [TODO])
       - standard numeric python types: int, long, float, Decimal
       - strings (like "0.09" or "2e-19")
       - booleans, including ``None`` (will leave them unchanged)
       - lists, sets or tuples containing any of the above

    If the argument is already a type that sympy understands, it will do
    nothing but return that value. This can be used at the beginning of a
    function to ensure you are working with the correct type.

    >>> from sympy import sympify

    >>> sympify(2).is_integer
    True
    >>> sympify(2).is_real
    True

    >>> sympify(2.0).is_real
    True
    >>> sympify("2.0").is_real
    True
    >>> sympify("2e-45").is_real
    True

    If the expression could not be converted, a SympifyError is raised.

    >>> sympify("x***2")
    Traceback (most recent call last):
    ...
    SympifyError: SympifyError: "could not parse u'x***2'"


    If the option ``strict`` is set to ``True``, only the types for which an
    explicit conversion has been defined are converted. In the other
    cases, a SympifyError is raised.

    >>> sympify(True)
    True
    >>> sympify(True, strict=True)
    Traceback (most recent call last):
    ...
    SympifyError: SympifyError: True

    To extend ``sympify`` to convert custom objects (not derived from ``Basic``),
    just define a ``_sympy_`` method to your class. You can do that even to
    classes that you do not own by subclassing or adding the method at runtime.

    >>> from sympy import Matrix
    >>> class MyList1(object):
    ...     def __iter__(self):
    ...         yield 1
    ...         yield 2
    ...         raise StopIteration
    ...     def __getitem__(self, i): return list(self)[i]
    ...     def _sympy_(self): return Matrix(self)
    >>> sympify(MyList1())
    [1]
    [2]

    If you do not have control over the class definition you could also use the
    ``converter`` global dictionary. The key is the class and the value is a
    function that takes a single argument and returns the desired SymPy
    object, e.g. ``converter[MyList] = lambda x: Matrix(x)``.

    >>> class MyList2(object):   # XXX Do not do this if you control the class!
    ...     def __iter__(self):  #     Use _sympy_!
    ...         yield 1
    ...         yield 2
    ...         raise StopIteration
    ...     def __getitem__(self, i): return list(self)[i]
    >>> from sympy.core.sympify import converter
    >>> converter[MyList2] = lambda x: Matrix(x)
    >>> sympify(MyList2())
    [1]
    [2]

    """
    try:
        cls = a.__class__
    except AttributeError:  #a is probably an old-style class object
        cls = type(a)
    if cls in sympy_classes:
        return a
    if cls in (bool, type(None)):
        if strict:
            raise SympifyError(a)
        else:
            return a

    try:
        return converter[cls](a)
    except KeyError:
        for superclass in getmro(cls):
            try:
                return converter[superclass](a)
            except KeyError:
                continue

    try:
        return a._sympy_()
    except AttributeError:
        pass

    if not isinstance(a, str):
        for coerce in (float, int):
            try:
                return sympify(coerce(a))
            except (TypeError, ValueError, AttributeError, SympifyError):
                continue

    if strict:
        raise SympifyError(a)

    if iterable(a):
        try:
            return type(a)([sympify(x, locals=locals, convert_xor=convert_xor,
                rational=rational) for x in a])
        except TypeError:
            # Not all iterables are rebuildable with their type.
            pass
    if isinstance(a, dict):
        try:
            return type(a)([sympify(x, locals=locals, convert_xor=convert_xor,
                rational=rational) for x in a.items()])
        except TypeError:
            # Not all iterables are rebuildable with their type.
            pass

    # At this point we were given an arbitrary expression
    # which does not inherit from Basic and doesn't implement
    # _sympy_ (which is a canonical and robust way to convert
    # anything to SymPy expression).
    #
    # As a last chance, we try to take "a"'s normal form via unicode()
    # and try to parse it. If it fails, then we have no luck and
    # return an exception
    try:
        a = str(a)
    except Exception as exc:
        raise SympifyError(a, exc)

    from sympy.parsing.sympy_parser import parse_expr, TokenError

    try:
        a = a.replace('\n', '')
        expr = parse_expr(a, locals or {}, rational, convert_xor)
    except (TokenError, SyntaxError):
        raise SympifyError('could not parse %r' % a)

    return expr

def _sympify(a):
    """Short version of sympify for internal usage for __add__ and __eq__
       methods where it is ok to allow some things (like Python integers
       and floats) in the expression. This excludes things (like strings)
       that are unwise to allow into such an expression.

       >>> from sympy import Integer
       >>> Integer(1) == 1
       True

       >>> Integer(1) == '1'
       False

       >>> from sympy import Symbol
       >>> from sympy.abc import x
       >>> x + 1
       x + 1

       >>> x + '1'
       Traceback (most recent call last):
           ...
       TypeError: unsupported operand type(s) for +: 'Symbol' and 'str'

       see: sympify
    """
    return sympify(a, strict=True)



"""
Continuous Random Variables - Prebuilt variables

Contains
========
Arcsin
Benini
Beta
BetaPrime
Cauchy
Chi
Dagum
Exponential
Gamma
Laplace
Logistic
LogNormal
Maxwell
Nakagami
Normal
Pareto
Rayleigh
StudentT
Triangular
Uniform
UniformSum
Weibull
WignerSemicircle
"""

from sympy import (exp, log, sqrt, pi, S, Dummy, Interval, S, sympify, gamma,
                   Piecewise, And, Eq, binomial, factorial, Sum, floor, Abs,
                   Symbol, log)
from sympy import beta as beta_fn
from .crv import SingleContinuousPSpace
from sympy.core.decorators import _sympifyit
import random

oo = S.Infinity

__all__ = ['ContinuousRV',
'Arcsin',
'Benini',
'Beta',
'BetaPrime',
'Cauchy',
'Chi',
'Dagum',
'Exponential',
'Gamma',
'Laplace',
'Logistic',
'LogNormal',
'Maxwell',
'Nakagami',
'Normal',
'Pareto',
'Rayleigh',
'StudentT',
'Triangular',
'Uniform',
'UniformSum',
'Weibull',
'WignerSemicircle'
]

def _value_check(condition, message):
    """
    Check a condition on input value.

    Raises ValueError with message if condition is not True
    """
    if condition is not True:
        raise ValueError(message)


def ContinuousRV(symbol, density, set=Interval(-oo,oo)):
    """
    Create a Continuous Random Variable given the following:

    -- a symbol
    -- a probability density function
    -- set on which the pdf is valid (defaults to entire real line)

    Returns a RandomSymbol.

    Many common continuous random variable types are already implemented.
    This function should be necessary only very rarely.

    Examples
    ========

    >>> from sympy import Symbol, sqrt, exp, pi
    >>> from sympy.stats import ContinuousRV, P, E

    >>> x = Symbol("x")

    >>> pdf = sqrt(2)*exp(-x**2/2)/(2*sqrt(pi)) # Normal distribution
    >>> X = ContinuousRV(x, pdf)

    >>> E(X)
    0
    >>> P(X>0)
    1/2
    """
    return SingleContinuousPSpace(symbol, density, set).value

########################################
# Continuous Probability Distributions #
########################################

#-------------------------------------------------------------------------------
# Arcsin distribution ----------------------------------------------------------

class ArcsinPSpace(SingleContinuousPSpace):
    def __new__(cls, name, a, b):
        a, b = sympify(a), sympify(b)
        x = Symbol(name)
        pdf = 1/(pi*sqrt((x-a)*(b-x)))
        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(a, b))
        return obj

def Arcsin(name, a=0, b=1):
    r"""
    Create a Continuous Random Variable with an arcsin distribution.

    The density of the arcsin distribution is given by

    .. math::
        f(x) := \frac{1}{\pi\sqrt{(x-a)(b-x)}}

    with :math:`x \in [a,b]`. It must hold that :math:`-\infty < a < b < \infty`.

    Parameters
    ==========

    a : Real number, the left interval boundary
    b : Real number, the right interval boundary

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Arcsin, density
    >>> from sympy import Symbol, simplify

    >>> a = Symbol("a", real=True)
    >>> b = Symbol("b", real=True)

    >>> X = Arcsin("x", a, b)

    >>> density(X)
    Lambda(_x, 1/(pi*sqrt((-_x + b)*(_x - a))))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Arcsine_distribution
    """

    return ArcsinPSpace(name, a, b).value

#-------------------------------------------------------------------------------
# Benini distribution ----------------------------------------------------------

class BeniniPSpace(SingleContinuousPSpace):
    def __new__(cls, name, alpha, beta, sigma):
        alpha, beta, sigma = sympify(alpha), sympify(beta), sympify(sigma)
        x = Symbol(name)
        pdf = (exp(-alpha*log(x/sigma)-beta*log(x/sigma)**2)
               *(alpha/x+2*beta*log(x/sigma)/x))
        obj = SingleContinuousPSpace.__new__(cls, x, pdf,
                                             set = Interval(sigma, oo))
        return obj

def Benini(name, alpha, beta, sigma):
    r"""
    Create a Continuous Random Variable with a Benini distribution.

    The density of the Benini distribution is given by

    .. math::
        f(x) := e^{-\alpha\log{\frac{x}{\sigma}}
                -\beta\log\left[{\frac{x}{\sigma}}\right]^2}
                \left(\frac{\alpha}{x}+\frac{2\beta\log{\frac{x}{\sigma}}}{x}\right)

    Parameters
    ==========

    alpha : Real number, `alpha` > 0 a shape
    beta : Real number, `beta` > 0 a shape
    sigma : Real number, `sigma` > 0 a scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Benini, density
    >>> from sympy import Symbol, simplify, pprint

    >>> alpha = Symbol("alpha", positive=True)
    >>> beta = Symbol("beta", positive=True)
    >>> sigma = Symbol("sigma", positive=True)

    >>> X = Benini("x", alpha, beta, sigma)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /                                                             2       \
          |   /                  /  x  \\             /  x  \            /  x  \|
          |   |        2*beta*log|-----||  - alpha*log|-----| - beta*log |-----||
          |   |alpha             \sigma/|             \sigma/            \sigma/|
    Lambda|x, |----- + -----------------|*e                                     |
          \   \  x             x        /                                       /

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Benini_distribution
    """

    return BeniniPSpace(name, alpha, beta, sigma).value

#-------------------------------------------------------------------------------
# Beta distribution ------------------------------------------------------------

class BetaPSpace(SingleContinuousPSpace):
    def __new__(cls, name, alpha, beta):
        alpha, beta = sympify(alpha), sympify(beta)

        _value_check(alpha > 0, "Alpha must be positive")
        _value_check(beta > 0, "Beta must be positive")

        x = Symbol(name)
        pdf = x**(alpha-1) * (1-x)**(beta-1) / beta_fn(alpha, beta)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(0, 1))
        obj.alpha = alpha
        obj.beta = beta
        return obj

    def sample(self):
        return {self.value: random.betavariate(self.alpha, self.beta)}

def Beta(name, alpha, beta):
    r"""
    Create a Continuous Random Variable with a Beta distribution.

    The density of the Beta distribution is given by

    .. math::
        f(x) := \frac{x^{\alpha-1}(1-x)^{\beta-1}} {\mathrm{B}(\alpha,\beta)}

    with :math:`x \in [0,1]`.

    Parameters
    ==========

    alpha : Real number, `alpha` > 0 a shape
    beta : Real number, `beta` > 0 a shape

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Beta, density, E, variance
    >>> from sympy import Symbol, simplify, pprint

    >>> alpha = Symbol("alpha", positive=True)
    >>> beta = Symbol("beta", positive=True)

    >>> X = Beta("x", alpha, beta)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /    alpha - 1         beta - 1                    \
          |   x         *(-x + 1)        *gamma(alpha + beta)|
    Lambda|x, -----------------------------------------------|
          \               gamma(alpha)*gamma(beta)           /

    >>> simplify(E(X, meijerg=True))
    alpha/(alpha + beta)

    >>> simplify(variance(X, meijerg=True))
    alpha*beta/((alpha + beta)**2*(alpha + beta + 1))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Beta_distribution
    [2] http://mathworld.wolfram.com/BetaDistribution.html
    """

    return BetaPSpace(name, alpha, beta).value

#-------------------------------------------------------------------------------
# Beta prime distribution ------------------------------------------------------

class BetaPrimePSpace(SingleContinuousPSpace):
    def __new__(cls, name, alpha, beta):
        alpha, beta = sympify(alpha), sympify(beta)
        x = Symbol(name)
        pdf = x**(alpha-1)*(1+x)**(-alpha-beta)/beta_fn(alpha, beta)
        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set = Interval(0, oo))
        return obj

def BetaPrime(name, alpha, beta):
    r"""
    Create a continuous random variable with a Beta prime distribution.

    The density of the Beta prime distribution is given by

    .. math::
        f(x) := \frac{x^{\alpha-1} (1+x)^{-\alpha -\beta}}{B(\alpha,\beta)}

    with :math:`x > 0`.

    Parameters
    ==========

    alpha : Real number, `alpha` > 0 a shape
    beta : Real number, `beta` > 0 a shape

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import BetaPrime, density
    >>> from sympy import Symbol, pprint

    >>> alpha = Symbol("alpha", positive=True)
    >>> beta = Symbol("beta", positive=True)

    >>> X = BetaPrime("x", alpha, beta)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /    alpha - 1        -alpha - beta                    \
          |   x         *(x + 1)             *gamma(alpha + beta)|
    Lambda|x, ---------------------------------------------------|
          \                 gamma(alpha)*gamma(beta)             /

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Beta_prime_distribution
    [2] http://mathworld.wolfram.com/BetaPrimeDistribution.html
    """

    return BetaPrimePSpace(name, alpha, beta).value

#-------------------------------------------------------------------------------
# Cauchy distribution ----------------------------------------------------------

class CauchyPSpace(SingleContinuousPSpace):
    def __new__(cls, name, x0, gamma):
        x0, gamma = sympify(x0), sympify(gamma)
        x = Symbol(name)
        pdf = 1/(pi*gamma*(1+((x-x0)/gamma)**2))
        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        return obj

def Cauchy(name, x0, gamma):
    r"""
    Create a continuous random variable with a Cauchy distribution.

    The density of the Cauchy distribution is given by

    .. math::
        f(x) := \frac{1}{\pi} \arctan\left(\frac{x-x_0}{\gamma}\right)
                +\frac{1}{2}

    Parameters
    ==========

    x0 : Real number, the location
    gamma : Real number, `gamma` > 0 the scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Cauchy, density
    >>> from sympy import Symbol

    >>> x0 = Symbol("x0")
    >>> gamma = Symbol("gamma", positive=True)

    >>> X = Cauchy("x", x0, gamma)

    >>> density(X)
    Lambda(_x, 1/(pi*gamma*(1 + (_x - x0)**2/gamma**2)))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Cauchy_distribution
    [2] http://mathworld.wolfram.com/CauchyDistribution.html
    """

    return CauchyPSpace(name, x0, gamma).value

#-------------------------------------------------------------------------------
# Chi distribution -------------------------------------------------------------

class ChiPSpace(SingleContinuousPSpace):
    def __new__(cls, name, k):
        k = sympify(k)
        x = Symbol(name)
        pdf = 2**(1-k/2)*x**(k-1)*exp(-x**2/2)/gamma(k/2)
        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set = Interval(0, oo))
        return obj

def Chi(name, k):
    r"""
    Create a continuous random variable with a Chi distribution.

    The density of the Chi distribution is given by

    .. math::
        f(x) := \frac{2^{1-k/2}x^{k-1}e^{-x^2/2}}{\Gamma(k/2)}

    with :math:`x \geq 0`.

    Parameters
    ==========

    k : Integer, `k` > 0 the number of degrees of freedom

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Chi, density, E, std
    >>> from sympy import Symbol, simplify

    >>> k = Symbol("k", integer=True)

    >>> X = Chi("x", k)

    >>> density(X)
    Lambda(_x, 2**(-k/2 + 1)*_x**(k - 1)*exp(-_x**2/2)/gamma(k/2))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Chi_distribution
    [2] http://mathworld.wolfram.com/ChiDistribution.html
    """

    return ChiPSpace(name, k).value

#-------------------------------------------------------------------------------
# Dagum distribution -----------------------------------------------------------

class DagumPSpace(SingleContinuousPSpace):
    def __new__(cls, name, p, a, b):
        p, a, b = sympify(p), sympify(a), sympify(b)
        x = Symbol(name)
        pdf = a*p/x*((x/b)**(a*p)/(((x/b)**a+1)**(p+1)))
        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        return obj

def Dagum(name, p, a, b):
    r"""
    Create a continuous random variable with a Dagum distribution.

    The density of the Dagum distribution is given by

    .. math::
        f(x) := \frac{a p}{x} \left( \frac{(\tfrac{x}{b})^{a p}}
                {\left((\tfrac{x}{b})^a + 1 \right)^{p+1}} \right)

    with :math:`x > 0`.

    Parameters
    ==========

    p : Real number, `p` > 0 a shape
    a : Real number, `a` > 0 a shape
    b : Real number, `b` > 0 a scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Dagum, density
    >>> from sympy import Symbol, simplify

    >>> p = Symbol("p", positive=True)
    >>> b = Symbol("b", positive=True)
    >>> a = Symbol("a", positive=True)

    >>> X = Dagum("x", p, a, b)

    >>> density(X)
    Lambda(_x, a*p*(_x/b)**(a*p)*((_x/b)**a + 1)**(-p - 1)/_x)

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Dagum_distribution
    """

    return DagumPSpace(name, p, a, b).value

#-------------------------------------------------------------------------------
# Exponential distribution -----------------------------------------------------

class ExponentialPSpace(SingleContinuousPSpace):
    def __new__(cls, name, rate):
        rate = sympify(rate)

        _value_check(rate > 0, "Rate must be positive.")

        x = Symbol(name)
        pdf = rate * exp(-rate*x)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(0, oo))
        obj.rate = rate
        return obj

    def sample(self):
        return {self.value: random.expovariate(self.rate)}

def Exponential(name, rate):
    r"""
    Create a continuous random variable with an Exponential distribution.

    The density of the exponential distribution is given by

    .. math::
        f(x) := \lambda \exp(-\lambda x)

    with :math:`x > 0`.

    Parameters
    ==========

    rate : Real number, `rate` > 0 the rate or inverse scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Exponential, density, cdf, E
    >>> from sympy.stats import variance, std, skewness
    >>> from sympy import Symbol

    >>> l = Symbol("lambda", positive=True)

    >>> X = Exponential("x", l)

    >>> density(X)
    Lambda(_x, lambda*exp(-_x*lambda))

    >>> cdf(X)
    Lambda(_z, Piecewise((1 - exp(-_z*lambda), _z >= 0), (0, True)))

    >>> E(X)
    1/lambda

    >>> variance(X)
    lambda**(-2)

    >>> skewness(X)
    2

    >>> X = Exponential('x', 10)

    >>> density(X)
    Lambda(_x, 10*exp(-10*_x))

    >>> E(X)
    1/10

    >>> std(X)
    1/10

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Exponential_distribution
    [2] http://mathworld.wolfram.com/ExponentialDistribution.html
    """

    return ExponentialPSpace(name, rate).value

#-------------------------------------------------------------------------------
# Gamma distribution -----------------------------------------------------------

class GammaPSpace(SingleContinuousPSpace):
    def __new__(cls, name, k, theta):
        k, theta = sympify(k), sympify(theta)

        _value_check(k > 0, "k must be positive")
        _value_check(theta > 0, "Theta must be positive")

        x = Symbol(name)
        pdf = x**(k-1) * exp(-x/theta) / (gamma(k)*theta**k)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(0, oo))
        obj.k = k
        obj.theta = theta
        return obj

    def sample(self):
        return {self.value: random.gammavariate(self.k, self.theta)}

def Gamma(name, k, theta):
    r"""
    Create a continuous random variable with a Gamma distribution.

    The density of the Gamma distribution is given by

    .. math::
        f(x) := \frac{1}{\Gamma(k) \theta^k} x^{k - 1} e^{-\frac{x}{\theta}}

    with :math:`x \in [0,1]`.

    Parameters
    ==========

    k : Real number, `k` > 0 a shape
    theta : Real number, `theta` > 0 a scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Gamma, density, cdf, E, variance
    >>> from sympy import Symbol, pprint

    >>> k = Symbol("k", positive=True)
    >>> theta = Symbol("theta", positive=True)

    >>> X = Gamma("x", k, theta)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /                     -x \
          |                   -----|
          |    k - 1      -k  theta|
          |   x     *theta  *e     |
    Lambda|x, ---------------------|
          \          gamma(k)      /

    >>> C = cdf(X, meijerg=True)
    >>> pprint(C, use_unicode=False)
          /   /                                   /     z  \            \
          |   |                       k*lowergamma|k, -----|            |
          |   |  k*lowergamma(k, 0)               \   theta/            |
    Lambda|z, <- ------------------ + ----------------------  for z >= 0|
          |   |     gamma(k + 1)           gamma(k + 1)                 |
          |   |                                                         |
          \   \                      0                        otherwise /

    >>> E(X)
    theta*gamma(k + 1)/gamma(k)

    >>> V = variance(X)
    >>> pprint(V, use_unicode=False)
           2      2                     -k      k + 1
      theta *gamma (k + 1)   theta*theta  *theta     *gamma(k + 2)
    - -------------------- + -------------------------------------
                2                           gamma(k)
           gamma (k)

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Gamma_distribution
    [2] http://mathworld.wolfram.com/GammaDistribution.html
    """

    return GammaPSpace(name, k, theta).value

#-------------------------------------------------------------------------------
# Laplace distribution ---------------------------------------------------------

class LaplacePSpace(SingleContinuousPSpace):
    def __new__(cls, name, mu, b):
        mu, b = sympify(mu), sympify(b)
        x = Symbol(name)
        pdf = 1/(2*b)*exp(-Abs(x-mu)/b)
        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        return obj

def Laplace(name, mu, b):
    r"""
    Create a continuous random variable with a Laplace distribution.

    The density of the Laplace distribution is given by

    .. math::
        f(x) := \frac{1}{2 b} \exp \left(-\frac{|x-\mu|}b \right)

    Parameters
    ==========

    mu : Real number, the location
    b : Real number, `b` > 0 a scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Laplace, density
    >>> from sympy import Symbol

    >>> mu = Symbol("mu")
    >>> b = Symbol("b", positive=True)

    >>> X = Laplace("x", mu, b)

    >>> density(X)
    Lambda(_x, exp(-Abs(_x - mu)/b)/(2*b))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Laplace_distribution
    [2] http://mathworld.wolfram.com/LaplaceDistribution.html
    """

    return LaplacePSpace(name, mu, b).value

#-------------------------------------------------------------------------------
# Logistic distribution --------------------------------------------------------

class LogisticPSpace(SingleContinuousPSpace):
    def __new__(cls, name, mu, s):
        mu, s = sympify(mu), sympify(s)

        x = Symbol(name)
        pdf = exp(-(x-mu)/s)/(s*(1+exp(-(x-mu)/s))**2)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        return obj

def Logistic(name, mu, s):
    r"""
    Create a continuous random variable with a logistic distribution.

    The density of the logistic distribution is given by

    .. math::
        f(x) := \frac{e^{-(x-\mu)/s}} {s\left(1+e^{-(x-\mu)/s}\right)^2}

    Parameters
    ==========

    mu : Real number, the location
    s : Real number, `s` > 0 a scale

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Logistic, density
    >>> from sympy import Symbol

    >>> mu = Symbol("mu", real=True)
    >>> s = Symbol("s", positive=True)

    >>> X = Logistic("x", mu, s)

    >>> density(X)
    Lambda(_x, exp((-_x + mu)/s)/(s*(exp((-_x + mu)/s) + 1)**2))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Logistic_distribution
    [2] http://mathworld.wolfram.com/LogisticDistribution.html
    """

    return LogisticPSpace(name, mu, s).value

#-------------------------------------------------------------------------------
# Log Normal distribution ------------------------------------------------------

class LogNormalPSpace(SingleContinuousPSpace):
    def __new__(cls, name, mean, std):
        mean, std = sympify(mean), sympify(std)

        x = Symbol(name)
        pdf = exp(-(log(x)-mean)**2 / (2*std**2)) / (x*sqrt(2*pi)*std)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(0, oo))
        obj.mean = mean
        obj.std = std
        return obj

    def sample(self):
        return {self.value: random.lognormvariate(self.mean, self.std)}

def LogNormal(name, mean, std):
    r"""
    Create a continuous random variable with a log-normal distribution.

    The density of the log-normal distribution is given by

    .. math::
        f(x) := \frac{1}{x\sqrt{2\pi\sigma^2}}
                e^{-\frac{\left(\ln x-\mu\right)^2}{2\sigma^2}}

    with :math:`x \geq 0`.

    Parameters
    ==========

    mu : Real number, the log-scale
    sigma : Real number, :math:`\sigma^2 > 0` a shape

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import LogNormal, density
    >>> from sympy import Symbol, simplify, pprint

    >>> mu = Symbol("mu", real=True)
    >>> sigma = Symbol("sigma", positive=True)

    >>> X = LogNormal("x", mu, sigma)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /                         2\
          |          -(-mu + log(x)) |
          |          ----------------|
          |                     2    |
          |     ___      2*sigma     |
          |   \/ 2 *e                |
    Lambda|x, -----------------------|
          |             ____         |
          \       2*x*\/ pi *sigma   /

    >>> X = LogNormal('x', 0, 1) # Mean 0, standard deviation 1

    >>> density(X)
    Lambda(_x, sqrt(2)*exp(-log(_x)**2/2)/(2*_x*sqrt(pi)))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Lognormal
    [2] http://mathworld.wolfram.com/LogNormalDistribution.html
    """

    return LogNormalPSpace(name, mean, std).value

#-------------------------------------------------------------------------------
# Maxwell distribution ---------------------------------------------------------

class MaxwellPSpace(SingleContinuousPSpace):
    def __new__(cls, name, a):
        a = sympify(a)

        x = Symbol(name)

        pdf = sqrt(2/pi)*x**2*exp(-x**2/(2*a**2))/a**3
        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set = Interval(0, oo))
        return obj

def Maxwell(name, a):
    r"""
    Create a continuous random variable with a Maxwell distribution.

    The density of the Maxwell distribution is given by

    .. math::
        f(x) := \sqrt{\frac{2}{\pi}} \frac{x^2 e^{-x^2/(2a^2)}}{a^3}

    with :math:`x \geq 0`.

    Parameters
    ==========

    a : Real number, `a` > 0

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Maxwell, density, E, variance
    >>> from sympy import Symbol, simplify

    >>> a = Symbol("a", positive=True)

    >>> X = Maxwell("x", a)

    >>> density(X)
    Lambda(_x, sqrt(2)*_x**2*exp(-_x**2/(2*a**2))/(sqrt(pi)*a**3))

    >>> E(X)
    2*sqrt(2)*a/sqrt(pi)

    >>> simplify(variance(X))
    a**2*(-8 + 3*pi)/pi

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Maxwell_distribution
    [2] http://mathworld.wolfram.com/MaxwellDistribution.html
    """

    return MaxwellPSpace(name, a).value

#-------------------------------------------------------------------------------
# Nakagami distribution --------------------------------------------------------

class NakagamiPSpace(SingleContinuousPSpace):
    def __new__(cls, name, mu, omega):
        mu, omega = sympify(mu), sympify(omega)
        x = Symbol(name)
        pdf = 2*mu**mu/(gamma(mu)*omega**mu)*x**(2*mu-1)*exp(-mu/omega*x**2)
        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set = Interval(0, oo))
        return obj

def Nakagami(name, mu, omega):
    r"""
    Create a continuous random variable with a Nakagami distribution.

    The density of the Nakagami distribution is given by

    .. math::
        f(x) := \frac{2\mu^\mu}{\Gamma(\mu)\omega^\mu} x^{2\mu-1}
                \exp\left(-\frac{\mu}{\omega}x^2 \right)

    with :math:`x > 0`.

    Parameters
    ==========

    mu : Real number, :math:`mu \geq \frac{1}{2}` a shape
    omega : Real number, `omega` > 0 the spread

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Nakagami, density, E, variance
    >>> from sympy import Symbol, simplify, pprint

    >>> mu = Symbol("mu", positive=True)
    >>> omega = Symbol("omega", positive=True)

    >>> X = Nakagami("x", mu, omega)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /                                2   \
          |                              -x *mu|
          |                              ------|
          |      2*mu - 1   mu      -mu  omega |
          |   2*x        *mu  *omega   *e      |
    Lambda|x, ---------------------------------|
          \               gamma(mu)            /

    >>> simplify(E(X, meijerg=True))
    sqrt(mu)*sqrt(omega)*gamma(mu + 1/2)/gamma(mu + 1)

    >>> V = simplify(variance(X, meijerg=True))
    >>> pprint(V, use_unicode=False)
          /                               2          \
    omega*\gamma(mu)*gamma(mu + 1) - gamma (mu + 1/2)/
    --------------------------------------------------
                 gamma(mu)*gamma(mu + 1)

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Nakagami_distribution
    """

    return NakagamiPSpace(name, mu, omega).value

#-------------------------------------------------------------------------------
# Normal distribution ----------------------------------------------------------

class NormalPSpace(SingleContinuousPSpace):
    def __new__(cls, name, mean, std):
        mean, std = sympify(mean), sympify(std)

        _value_check(std > 0, "Standard deviation must be positive")

        x = Symbol(name)
        pdf = exp(-(x-mean)**2 / (2*std**2)) / (sqrt(2*pi)*std)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        obj.mean = mean
        obj.std = std
        obj.variance = std**2
        return obj

    def sample(self):
        return {self.value: random.normalvariate(self.mean, self.std)}

def Normal(name, mean, std):
    r"""
    Create a continuous random variable with a Normal distribution.

    The density of the Normal distribution is given by

    .. math::
        f(x) := \frac{1}{\sigma\sqrt{2\pi}} e^{ -\frac{(x-\mu)^2}{2\sigma^2} }

    Parameters
    ==========

    mu : Real number, the mean
    sigma : Real number, :math:`\sigma^2 > 0` the variance

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Normal, density, E, std, cdf, skewness
    >>> from sympy import Symbol, simplify, pprint

    >>> mu = Symbol("mu")
    >>> sigma = Symbol("sigma", positive=True)

    >>> X = Normal("x", mu, sigma)

    >>> density(X)
    Lambda(_x, sqrt(2)*exp(-(_x - mu)**2/(2*sigma**2))/(2*sqrt(pi)*sigma))

    >>> C = simplify(cdf(X))
    >>> pprint(C, use_unicode=False)
              /      /  ___         \    \
              |      |\/ 2 *(z - mu)|    |
              |   erf|--------------|    |
              |      \   2*sigma    /   1|
        Lambda|z, ------------------- + -|
              \            2            2/

    >>> simplify(skewness(X))
    0

    >>> X = Normal("x", 0, 1) # Mean 0, standard deviation 1
    >>> density(X)
    Lambda(_x, sqrt(2)*exp(-_x**2/2)/(2*sqrt(pi)))

    >>> E(2*X + 1)
    1

    >>> simplify(std(2*X + 1))
    2

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Normal_distribution
    [2] http://mathworld.wolfram.com/NormalDistributionFunction.html
    """

    return NormalPSpace(name, mean, std).value

#-------------------------------------------------------------------------------
# Pareto distribution ----------------------------------------------------------

class ParetoPSpace(SingleContinuousPSpace):
    def __new__(cls, name, xm, alpha):
        xm, alpha = sympify(xm), sympify(alpha)

        _value_check(xm > 0, "Xm must be positive")
        _value_check(alpha > 0, "Alpha must be positive")

        x = Symbol(name)
        pdf = alpha * xm**alpha / x**(alpha+1)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(xm, oo))
        obj.xm = xm
        obj.alpha = alpha
        return obj

    def sample(self):
        return {self.value: random.paretovariate(self.alpha)}

def Pareto(name, xm, alpha):
    r"""
    Create a continuous random variable with the Pareto distribution.

    The density of the Pareto distribution is given by

    .. math::
        f(x) := \frac{\alpha\,x_\mathrm{m}^\alpha}{x^{\alpha+1}}

    with :math:`x \in [x_m,\infty]`.

    Parameters
    ==========

    xm : Real number, `xm` > 0 a scale
    alpha : Real number, `alpha` > 0 a shape

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Pareto, density
    >>> from sympy import Symbol

    >>> xm = Symbol("xm", positive=True)
    >>> beta = Symbol("beta", positive=True)

    >>> X = Pareto("x", xm, beta)

    >>> density(X)
    Lambda(_x, _x**(-beta - 1)*beta*xm**beta)

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Pareto_distribution
    [2] http://mathworld.wolfram.com/ParetoDistribution.html
    """

    return ParetoPSpace(name, xm, alpha).value

#-------------------------------------------------------------------------------
# Rayleigh distribution --------------------------------------------------------

class RayleighPSpace(SingleContinuousPSpace):
    def __new__(cls, name, sigma):
        sigma = sympify(sigma)
        x = Symbol(name)
        pdf = x/sigma**2*exp(-x**2/(2*sigma**2))
        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set = Interval(0, oo))
        return obj

def Rayleigh(name, sigma):
    r"""
    Create a continuous random variable with a Rayleigh distribution.

    The density of the Rayleigh distribution is given by

    .. math ::
        f(x) := \frac{x}{\sigma^2} e^{-x^2/2\sigma^2}

    with :math:`x > 0`.

    Parameters
    ==========

    sigma : Real number, `sigma` > 0

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Rayleigh, density, E, variance
    >>> from sympy import Symbol, simplify

    >>> sigma = Symbol("sigma", positive=True)

    >>> X = Rayleigh("x", sigma)

    >>> density(X)
    Lambda(_x, _x*exp(-_x**2/(2*sigma**2))/sigma**2)

    >>> E(X)
    sqrt(2)*sqrt(pi)*sigma/2

    >>> variance(X)
    -pi*sigma**2/2 + 2*sigma**2

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Rayleigh_distribution
    [2] http://mathworld.wolfram.com/RayleighDistribution.html
    """

    return RayleighPSpace(name, sigma).value

#-------------------------------------------------------------------------------
# StudentT distribution --------------------------------------------------------

class StudentTPSpace(SingleContinuousPSpace):
    def __new__(cls, name, nu):
        nu = sympify(nu)
        x = Symbol(name)
        pdf = 1/(sqrt(nu)*beta_fn(S(1)/2,nu/2))*(1+x**2/nu)**(-(nu+1)/2)
        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        return obj

def StudentT(name, nu):
    r"""
    Create a continuous random variable with a student's t distribution.

    The density of the student's t distribution is given by

    .. math::
        f(x) := \frac{\Gamma \left(\frac{\nu+1}{2} \right)}
                {\sqrt{\nu\pi}\Gamma \left(\frac{\nu}{2} \right)}
                \left(1+\frac{x^2}{\nu} \right)^{-\frac{\nu+1}{2}}

    Parameters
    ==========

    nu : Real number, `nu` > 0, the degrees of freedom

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import StudentT, density, E, variance
    >>> from sympy import Symbol, simplify, pprint

    >>> nu = Symbol("nu", positive=True)

    >>> X = StudentT("x", nu)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /             nu   1              \
          |           - -- - -              |
          |             2    2              |
          |   / 2    \                      |
          |   |x     |              /nu   1\|
          |   |-- + 1|        *gamma|-- + -||
          |   \nu    /              \2    2/|
    Lambda|x, ------------------------------|
          |        ____   ____      /nu\    |
          |      \/ pi *\/ nu *gamma|--|    |
          \                         \2 /    /

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Student_t-distribution
    [2] http://mathworld.wolfram.com/Studentst-Distribution.html
    """

    return StudentTPSpace(name, nu).value

#-------------------------------------------------------------------------------
# Triangular distribution ------------------------------------------------------

class TriangularPSpace(SingleContinuousPSpace):
    def __new__(cls, name, a, b, c):
        a, b, c = sympify(a), sympify(b), sympify(c)

        x = Symbol(name)
        pdf = Piecewise(
                (2*(x-a)/((b-a)*(c-a)), And(a<=x, x<c)),
                (2/(b-a), Eq(x,c)),
                (2*(b-x)/((b-a)*(b-c)), And(c<x, x<=b)),
                (S.Zero, True))

        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        return obj

def Triangular(name, a, b, c):
    r"""
    Create a continuous random variable with a triangular distribution.

    The density of the triangular distribution is given by

    .. math::
        f(x) := \begin{cases}
                  0 & \mathrm{for\ } x < a, \\
                  \frac{2(x-a)}{(b-a)(c-a)} & \mathrm{for\ } a \le x < c, \\
                  \frac{2}{b-a} & \mathrm{for\ } x = c, \\
                  \frac{2(b-x)}{(b-a)(b-c)} & \mathrm{for\ } c < x \le b, \\
                  0 & \mathrm{for\ } b < x.
                \end{cases}

    Parameters
    ==========

    a : Real number, :math:`a \in \left(-\infty, \infty\right)`
    b : Real number, :math:`a < b`
    c : Real number, :math:`a \leq c \leq b`

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Triangular, density, E
    >>> from sympy import Symbol

    >>> a = Symbol("a")
    >>> b = Symbol("b")
    >>> c = Symbol("c")

    >>> X = Triangular("x", a,b,c)

    >>> density(X)
    Lambda(_x, Piecewise(((2*_x - 2*a)/((-a + b)*(-a + c)),
                        And(_x < c, a <= _x)),
                        (2/(-a + b), _x == c),
                        ((-2*_x + 2*b)/((-a + b)*(b - c)),
                        And(_x <= b, c < _x)),
                        (0, True)))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Triangular_distribution
    [2] http://mathworld.wolfram.com/TriangularDistribution.html
    """

    return TriangularPSpace(name, a, b, c).value

#-------------------------------------------------------------------------------
# Uniform distribution ---------------------------------------------------------

class UniformPSpace(SingleContinuousPSpace):
    def __new__(cls, name, left, right):
        left, right = sympify(left), sympify(right)

        x = Symbol(name)
        pdf = Piecewise(
                (S.One/(right-left), And(left <= x, x <= right)),
                (S.Zero, True))

        obj = SingleContinuousPSpace.__new__(cls, x, pdf)
        obj.left = left
        obj.right = right
        return obj

    def compute_cdf(self, expr, **kwargs):
        from sympy import Lambda, Min
        z = Dummy('z', real=True, bounded=True)
        result = SingleContinuousPSpace.compute_cdf(self, expr, **kwargs)
        result = result(z).subs({Min(z, self.right): z,
                                 Min(z, self.left, self.right): self.left})
        return Lambda(z, result)

    def integrate(self, expr, rvs=None, **kwargs):
        from sympy import Max, Min
        result = SingleContinuousPSpace.integrate(self, expr, rvs, **kwargs)
        result = result.subs({Max(self.left, self.right): self.right,
                              Min(self.left, self.right): self.left})
        return result

    def sample(self):
        return {self.value: random.uniform(self.left, self.right)}

def Uniform(name, left, right):
    r"""
    Create a continuous random variable with a uniform distribution.

    The density of the uniform distribution is given by

    .. math::
        f(x) := \begin{cases}
                  \frac{1}{b - a} & \text{for } x \in [a,b]  \\
                  0               & \text{otherwise}
                \end{cases}

    with :math:`x \in [a,b]`.

    Parameters
    ==========

    a : Real number, :math:`-\infty < a` the left boundary
    b : Real number, :math:`a < b < \infty` the right boundary

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Uniform, density, cdf, E, variance, skewness
    >>> from sympy import Symbol, simplify

    >>> a = Symbol("a")
    >>> b = Symbol("b")

    >>> X = Uniform("x", a, b)

    >>> density(X)
    Lambda(_x, Piecewise((1/(-a + b), And(_x <= b, a <= _x)), (0, True)))

    >>> cdf(X)
    Lambda(_z, _z/(-a + b) - a/(-a + b))

    >>> simplify(E(X))
    a/2 + b/2

    >>> simplify(variance(X))
    a**2/12 - a*b/6 + b**2/12

    >>> simplify(skewness(X))
    0

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Uniform_distribution_%28continuous%29
    [2] http://mathworld.wolfram.com/UniformDistribution.html
    """

    return UniformPSpace(name, left, right).value

#-------------------------------------------------------------------------------
# UniformSum distribution ------------------------------------------------------

class UniformSumPSpace(SingleContinuousPSpace):
    def __new__(cls, name, n):
        n = sympify(n)

        x = Symbol(name)
        k = Dummy("k")
        pdf =1/factorial(n-1)*Sum((-1)**k*binomial(n,k)*(x-k)**(n-1), (k,0,floor(x)))

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(0,n))
        return obj

def UniformSum(name, n):
    r"""
    Create a continuous random variable with an Irwin-Hall distribution.

    The probability distribution function depends on a single parameter
    `n` which is an integer.

    The density of the Irwin-Hall distribution is given by

    .. math ::
        f(x) := \frac{1}{(n-1)!}\sum_{k=0}^{\lfloor x\rfloor}(-1)^k
                \binom{n}{k}(x-k)^{n-1}

    Parameters
    ==========

    n : Integral number, `n` > 0

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import UniformSum, density
    >>> from sympy import Symbol, pprint

    >>> n = Symbol("n", integer=True)

    >>> X = UniformSum("x", n)

    >>> D = density(X)
    >>> pprint(D, use_unicode=False)
          /   floor(x)                        \
          |     ___                           |
          |     \  `                          |
          |      \         k         n - 1 /n\|
          |       )    (-1) *(-k + x)     *| ||
          |      /                         \k/|
          |     /__,                          |
          |    k = 0                          |
    Lambda|x, --------------------------------|
          \               (n - 1)!            /

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Uniform_sum_distribution
    [2] http://mathworld.wolfram.com/UniformSumDistribution.html
    """

    return UniformSumPSpace(name, n).value

#-------------------------------------------------------------------------------
# Weibull distribution ---------------------------------------------------------

class WeibullPSpace(SingleContinuousPSpace):
    def __new__(cls, name, alpha, beta):
        alpha, beta = sympify(alpha), sympify(beta)

        _value_check(alpha > 0, "Alpha must be positive")
        _value_check(beta > 0, "Beta must be positive")

        x = Symbol(name)
        pdf = beta * (x/alpha)**(beta-1) * exp(-(x/alpha)**beta) / alpha

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set=Interval(0, oo))
        obj.alpha = alpha
        obj.beta = beta
        return obj

    def sample(self):
        return {self.value: random.weibullvariate(self.alpha, self.beta)}

def Weibull(name, alpha, beta):
    r"""
    Create a continuous random variable with a Weibull distribution.

    The density of the Weibull distribution is given by

    .. math::
        f(x) := \begin{cases}
                  \frac{k}{\lambda}\left(\frac{x}{\lambda}\right)^{k-1}
                  e^{-(x/\lambda)^{k}} & x\geq0\\
                  0 & x<0
                \end{cases}

    Parameters
    ==========

    lambda : Real number, :math:`\lambda > 0` a scale
    k : Real number, `k` > 0 a shape

    Returns
    =======

    A RandomSymbol.

    Examples
    ========

    >>> from sympy.stats import Weibull, density, E, variance
    >>> from sympy import Symbol, simplify

    >>> l = Symbol("lambda", positive=True)
    >>> k = Symbol("k", positive=True)

    >>> X = Weibull("x", l, k)

    >>> density(X)
    Lambda(_x, k*(_x/lambda)**(k - 1)*exp(-(_x/lambda)**k)/lambda)

    >>> simplify(E(X))
    lambda*gamma(1 + 1/k)

    >>> simplify(variance(X))
    lambda**2*(-gamma(1 + 1/k)**2 + gamma(1 + 2/k))

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Weibull_distribution
    [2] http://mathworld.wolfram.com/WeibullDistribution.html

    """

    return WeibullPSpace(name, alpha, beta).value

#-------------------------------------------------------------------------------
# Wigner semicircle distribution -----------------------------------------------

class WignerSemicirclePSpace(SingleContinuousPSpace):
    def __new__(cls, name, R):
        R = sympify(R)

        x = Symbol(name)
        pdf = 2/(pi*R**2)*sqrt(R**2-x**2)

        obj = SingleContinuousPSpace.__new__(cls, x, pdf, set = Interval(-R, R))
        return obj

def WignerSemicircle(name, R):
    r"""
    Create a continuous random variable with a Wigner semicircle distribution.

    The density of the Wigner semicircle distribution is given by

    .. math::
        f(x) := \frac2{\pi R^2}\,\sqrt{R^2-x^2}

    with :math:`x \in [-R,R]`.

    Parameters
    ==========

    R : Real number, `R` > 0 the radius

    Returns
    =======

    A `RandomSymbol`.

    Examples
    ========

    >>> from sympy.stats import WignerSemicircle, density, E
    >>> from sympy import Symbol, simplify

    >>> R = Symbol("R", positive=True)

    >>> X = WignerSemicircle("x", R)

    >>> density(X)
    Lambda(_x, 2*sqrt(-_x**2 + R**2)/(pi*R**2))

    >>> E(X)
    0

    References
    ==========

    [1] http://en.wikipedia.org/wiki/Wigner_semicircle_distribution
    [2] http://mathworld.wolfram.com/WignersSemicircleLaw.html
    """

    return WignerSemicirclePSpace(name, R).value



"""Implementation of :class:`PythonComplexDomain` class. """

from sympy.polys.domains.realdomain import RealDomain

from sympy.polys.polyerrors import CoercionFailed

class PythonComplexDomain(RealDomain): # XXX: tmp solution
    """Complex domain. """

    rep   = 'CC'

    dtype = complex
    zero  = dtype(0)
    one   = dtype(1)
    alias = 'CC_python'

    def __init__(self):
        pass

    def to_sympy(self, a):
        """Convert `a` to a SymPy object. """
        return sympy_mpf(a)

    def from_sympy(self, a):
        """Convert SymPy's Integer to `dtype`. """
        b = a.evalf()

        if b.is_Float and b not in [S.Infinity, S.NegativeInfinity]:
            return float(b)
        else:
            raise CoercionFailed("expected Float object, got %s" % a)

    def from_ZZ_python(K1, a, K0):
        """Convert a Python `int` object to `dtype`. """
        return K1.dtype(a)

    def from_QQ_python(K1, a, K0):
        """Convert a Python `Fraction` object to `dtype`. """
        return K1.dtype(a.numerator) / a.denominator

    def from_ZZ_sympy(K1, a, K0):
        """Convert a SymPy `Integer` object to `dtype`. """
        return K1.dtype(a.p)

    def from_QQ_sympy(K1, a, K0):
        """Convert a SymPy `Rational` object to `dtype`. """
        return K1.dtype(a.p) / a.q

    def from_ZZ_gmpy(K1, a, K0):
        """Convert a GMPY `mpz` object to `dtype`. """
        return K1.dtype(int(a))

    def from_QQ_gmpy(K1, a, K0):
        """Convert a GMPY `mpq` object to `dtype`. """
        return K1.dtype(int(a.numer())) / int(a.denom)

    def from_RR_sympy(K1, a, K0):
        """Convert a SymPy `Float` object to `dtype`. """
        return K1.dtype(a)

    def from_RR_mpmath(K1, a, K0):
        """Convert a mpmath `mpf` object to `dtype`. """
        return K1.dtype(a)

    def from_FF_float(K1, a, K0):
        return K1.dtype(a)

    def real(self, a):
        return a.real

    def imag(self, a):
        return a.imag
